//
//
//

#include "undopbc.h"

#include <lpmd/simulation.h>
#include <lpmd/simulationhistory.h>

#include <iostream>
#include <iomanip>

using namespace lpmd;

UndoPBCModifier::UndoPBCModifier(std::string args): Plugin("undopbc", "2.0"), oldpositions(0)
{
 ParamList & params = (*this);
 //
 DefineKeyword("start", "0");
 DefineKeyword("end", "-1");
 DefineKeyword("each", "1");
 // hasta aqui los valores por omision
 ProcessArguments(args);
 start = int(params["start"]);
 end = int(params["end"]);
 each = int(params["each"]);
 first_apply = true;
}

UndoPBCModifier::~UndoPBCModifier() { delete [] oldpositions; }

void UndoPBCModifier::ShowHelp() const
{
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
 std::cout << " Module Name        = undopbc                                                  \n";
 std::cout << " Problems Report to = admin@lpmd.cl                                            \n";
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
 std::cout << " General Info      >>                                                          \n";
 std::cout << "      This plugin is used to remove periodic boundary conditions in a set      \n";
 std::cout << "      of multiple configurations.                                              \n";
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
 std::cout << " General Options   >>                                                          \n";
 std::cout << "      start         : Determines in which step the plugin begins to be applied.\n";
 std::cout << "      end           : Determines in which step the plugin ceases to be applied.\n";
 std::cout << "      each          : Determines how often (each how many time-steps) the      \n";
 std::cout << "                      plugin must be applied.                                  \n";
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
 std::cout << " Example           >>                                                          \n";
 std::cout << " #Load the plugin  :                                                           \n";
 std::cout << " use undopbc                                                                   \n";
 std::cout << " enduse                                                                        \n";
 std::cout << " #Applying the plugin :                                                        \n";
 std::cout << " apply undopbc start=0 each=1 end=100                                        \n\n";
 std::cout << "      The plugin is used to turn off the periodic boundary conditions for the  \n";
 std::cout << "      first 100 time steps.                                                    \n";
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
}

void UndoPBCModifier::Apply(Simulation & sim)
{
 lpmd::BasicParticleSet & atoms = sim.Atoms();
 lpmd::BasicCell & cell = sim.Cell();
 if (first_apply)
 {
  oldpositions = new Vector[atoms.Size()];
  for (long int i=0;i<atoms.Size();++i){ oldpositions[i] = atoms[i].Position(); /*std::cout << "Old pos. " << i << " " << oldpositions[i] << "\n"; */}
  first_apply = false;
 }
 else
 {
  for (long int i=0;i<atoms.Size();++i)
  {
//   std::cout << "Moving " << i << " : " << atoms[i].Position() << " to " <<  oldpositions[i] << " + *** ENTRO a Disp *** " <<  cell.Displacement(oldpositions[i], atoms[i].Position()) ;
   atoms[i].Position() = oldpositions[i] + cell.Displacement(oldpositions[i], atoms[i].Position());
  // std::cout << "*** SALGO de Disp *** = " << atoms[i].Position() << "\n";
  }
 }
}

/*
 //
 // Undo periodicity 
 //
 if (zerocm) ZeroCM(hist[0]);
 StoredConfiguration scratch(hist[0]);
 Vector ** noperiodic = new Vector*[N];
 for (int t=0;t<N;++t) noperiodic[t] = new Vector[nat];
 BasicParticleSet & scratch_atoms = scratch.Atoms();
 BasicCell & cell = scratch.Cell();
 for (long int i=0;i<nat;++i)
 {
  noperiodic[0][i] = scratch_atoms[i].Position();
 }
 for (int t=1;t<N;++t)
 {
  if (zerocm) ZeroCM(hist[t]);
  DebugStream() << "-> MSD: Undoing periodicity, configuration " << t << '\n';
  for (long int i=0;i<nat;++i)
  {
   const Vector & v0 = scratch_atoms[0].Position() = hist[t-1].Atoms()[i].Position();
   const Vector & v1 = scratch_atoms[1].Position() = hist[t].Atoms()[i].Position();
   noperiodic[t][i] = noperiodic[t-1][i] + cell.Displacement(v0, v1);
  }
 }

 //
 //
 //

*/


// Esto se incluye para que el modulo pueda ser cargado dinamicamente
Plugin * create(std::string args) { return new UndoPBCModifier(args); }
void destroy(Plugin * m) { delete m; }

